/*
 *  Kylin security execute control
 *
 *  This file contains attributes manage apis for Kylin security exectl
 *
 *  Authors:  songchuanfei <songchuanfei@kylinos.cn>
 *  Authors:  huanfuxi <huanfuxi@kylinos.cn>
 *  Authors:  mengyuan <mengyuan@kylinos.cn>
 *
 *  Copyright (C) 2017,Tianjin KYLIN Information Technology Co., Ltd.
 *
 *  This library is free software; you can redistribute it and/or
 *  modify it under the terms of the GNU Lesser General Public
 *  License as published by the Free Software Foundation; either
 *  version 2 of the License, or (at your option) any later version.
 *
 *  This library is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 *  Lesser General Public License for more details.
 *
 *  You should have received a copy of the GNU Lesser General Public
 *  License along with this library; if not, write to the
 *  Free Software Foundation, Inc., 51 Franklin St, Fifth Floor,
 *  Boston, MA 02110-1301, USA.
 */
#ifndef LIBRARY_KYSECURITY_DBUS_H
#define LIBRARY_KYSECURITY_DBUS_H

#include <stdio.h>
#include <unistd.h>
#include <stdlib.h>
#include <sys/types.h>
#include <sys/stat.h>
#include <attr/xattr.h>
#include <string.h>
#include <errno.h>
#include <limits.h>

#define KYSEC_DEFAULT_LABEL_LEN	32
typedef struct kysec_mark_record{
    char path[PATH_MAX];
    char label[KYSEC_DEFAULT_LABEL_LEN];
}kysec_mark_record;

typedef struct kysec_dpkg_record{
    char path[PATH_MAX];
    char label[KYSEC_DEFAULT_LABEL_LEN];
    int file_type;
}kysec_dpkg_record;

#ifdef __cplusplus
extern "C" {
#endif

/*
 * set kysec label to file.
 * return values:
 * 0 : success
 * other : failed
 */
int kysec_label_set(char *path, char *label);

/*
 * set kysec protect label to file.
 * return values:
 * 0 : success
 * other : failed
 */
int kysec_protect_set(char *path, char *protect);

/*
 * set kysec identify label to file.
 * return values:
 * 0 : success
 * other : failed
 */
int kysec_identify_set(char *path, char *identify);

/*
 * set kysec exectl label to file.
 * return values:
 * 0 : success
 * other : failed
 */
int kysec_exectl_set(char *path, char *exectl);

/*
 * multi set kysec protect label to file.
 * return values:
 * < 0 : failed
 * >= 0 : success count
 */
int kysec_protect_multi_set(kysec_mark_record *list, int count);

/*
 * multi set kysec identify label to file.
 * return values:
 * < 0 : failed
 * >= 0 : success count
 */
int kysec_identify_multi_set(kysec_mark_record *list, int count);

/*
 * multi set kysec exectl label to file.
 * return values:
 * < 0 : failed
 * >= 0 : success count
 */
int kysec_exectl_multi_set(kysec_mark_record *list, int count);

/*
 * multi set kysec exectl label to file for dpkg.
 * return values:
 * < 0 : failed
 * >= 0 : success count
 */
int kysec_exectl_multi_set_for_dpkg(kysec_dpkg_record *list, int count);

/*
 * check username and passwd is matched.
 * return values:
 * 0 : matched
 * other : not matched
 */
int kysec_check_user_valid(const char *uname, const char *pwd);

char *kysec_calc_file_hash_value(char *path);

/*
批量计算文件hash值
失败返回NULL； 成功返回file_list相同大小hash值列表
*/
char **kysec_calc_multi_file_hash_value(char *file_list[], int count);

char *kysec_get_file_hash_value(char *path);

/*
批量获取数据库中文件hash值
失败返回NULL； 成功返回file_list相同大小hash值列表
*/
char **kysec_get_multi_file_hash_value(char *file_list[], int count);

#ifdef __cplusplus
}
#endif

#endif
